package com.andrewlalis.perfin.view.component.module;

import com.andrewlalis.perfin.model.Profile;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.chart.*;
import javafx.scene.layout.Pane;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * A module for visualizing the total asset value in the user's profile over
 * a configurable period of time.
 */
public class TotalAssetsGraphModule extends DashboardModule {
    private final ObservableList<XYChart.Data<String, Number>> totalAssetDataPoints = FXCollections.observableArrayList();

    public TotalAssetsGraphModule(Pane parent) {
        super(parent);
        Axis<String> xAxis = new CategoryAxis();
        Axis<Number> yAxis = new NumberAxis();

        LineChart<String, Number> chart = new LineChart<>(xAxis, yAxis, FXCollections.observableArrayList(
                new XYChart.Series<>("Total Assets", totalAssetDataPoints)
        ));
        chart.setLegendVisible(false);
        this.getChildren().add(new ModuleHeader(
                "Total Assets over Time"
        ));
        this.getChildren().add(chart);
    }

    @Override
    public void refreshContents() {
        totalAssetDataPoints.clear();
        String[] dateLabels = new String[12];
        double[] values = new double[12];
        CompletableFuture<?>[] futures = new CompletableFuture[12];
        for (int i = 0; i < 12; i++) {
            final int idx = i;
            Instant timestamp = Instant.now().minus((12 - i - 1) * 30, ChronoUnit.DAYS);
            dateLabels[i] = LocalDate.from(timestamp.atZone(ZoneId.systemDefault())).toString();
            futures[i] = Profile.getCurrent().dataSource().getCombinedAccountBalances(timestamp)
                    .thenAccept(moneyValues -> {
                        values[idx] = moneyValues.getFirst().amount().doubleValue();
                    });
        }
        CompletableFuture.allOf(futures).thenRun(() -> {
            List<XYChart.Data<String, Number>> dataPoints = new ArrayList<>(12);
            for (int i = 0; i < 12; i++) {
                dataPoints.add(new XYChart.Data<>(dateLabels[i], values[i]));
            }
            Platform.runLater(() -> totalAssetDataPoints.addAll(dataPoints));
        });
    }
}
